# -*- coding: utf-8; -*-
#
# index.rb 007
#
# Copyright (C) 2001-2009, TADA Tadashi <t@tdtds.jp>
# You can redistribute it and/or modify it under GPL2.
#
# modified by teruo Kohashi, teru@kohashi.org www.kohashi.org/dev/blex.html
# 2012-03-01 : $blex_start_always=true, display start page in latest mode

BEGIN { $stdout.binmode }
begin
	Encoding::default_external = 'UTF-8'
rescue NameError
	$KCODE = 'n'
end

begin
	if FileTest::symlink?( __FILE__ ) then
		org_path = File::dirname( File::readlink( __FILE__ ) ).untaint
	else
		org_path = File::dirname( __FILE__ ).untaint
	end
	$:.unshift( org_path ) unless $:.include?( org_path )
	require 'tdiary'

	encoding_error = {}
	cgi = CGI::new(:accept_charset => "UTF-8") do |name, value|
		encoding_error[name] = value
	end
	if encoding_error.empty?
		@cgi = cgi
	else
		@cgi = CGI::new(:accept_charset => 'shift_jis')
		@cgi.params = cgi.params
	end
	
	### dispatch for blex ##########################
	def get00_menu_tree
		menu = BLEX_MENU
		return nil if menu.nil?
		menu_tree = []
		menu.each_line {|line|
			m = line.split(nil)
			next if m.empty?
			label = m[0]
			link = m[1]
			cats = m[2..-1]
			menu_tree <<  [ label, link, cats ]
		}
		menu_tree
	end

	def get00_member_cats( cat )
		return [] if cat.nil?
		return [] if cat.empty?
		menu = get00_menu_tree
		return [ cat ] if menu.nil? or menu.empty?
		result = []
		menu.each { |i|
			if i[1] == cat
				result |= i[2] if i[2]
#			elsif i[2] and i[2].include?( cat )
#				result << cat
			end
		}
		result << cat
		result
	end
	
	cat = @cgi.params['c'][0]
	if cat
		@cgi.params['blogcategory'] = [ cat ]
		@cgi.params.delete('c')
	else
		cat = @cgi.params['blogcategory'][0]
	end
	day = @cgi.params['d'][0]
	if day
		@cgi.params.delete('d')
		@cgi.params['date'] = [ day ]
	else
		day = @cgi.params['date'][0]
	end
	if day and /^\d{9,10}$/ =~ day
		day = Time.at( day.to_i ).strftime('%Y%m%d')
		@cgi.params['date'] = [ day ] 
	end
	
	if day and @cgi.params['n'][0]
		@cgi.params['date'] = [ day + '-10' ]
	elsif day.nil? and File.exists?( "#{DATA_PATH}/cache/blog_category" )
		# init
		index_catdays = {}
		PStore.new( "#{DATA_PATH}/cache/blog_category" ).transaction do |db|
			index_catdays.update(db['blog_category'])
			db.abort
		end
		# no cats, set start page and the start page is exist, show the start page
		if $blex_start_always and cat.nil? and $blex_start and index_catdays[$blex_start] and index_catdays[$blex_start].keys.size == 1
			@cgi.params['blogcategory'] = [ $blex_start ]
			@cgi.params['date'] = [ index_catdays[$blex_start].keys.first ]
		# no params, set start page and the start page is exit, show the start page
		elsif @cgi.params.empty? and $blex_start and index_catdays[$blex_start] and index_catdays[$blex_start].keys.size == 1
			@cgi.params['blogcategory'] = [ $blex_start ]
			@cgi.params['date'] = [ index_catdays[$blex_start].keys.first ]
		# nth -> latest
		elsif cat.nil?
		
		# single page cat, show the cat page
		elsif index_catdays[cat] and index_catdays[cat].keys.size == 1 and @cgi.params['n'][0].nil?
			day = index_catdays[cat].keys.first
			@cgi.params['date'] = [ day ]
		# date nortation, show the date page
		elsif /^(\d{8}|\d{4}|\d{6}|\d{8}-\d+)$/ =~ cat
			@cgi.params['date'] = [ cat ]
			@cgi.params.delete('blogcategory')
		elsif /^\d{9,10}$/ =~ cat
			@cgi.params['date'] = [ Time.at( cat.to_i ).strftime('%Y%m%d') ]
			@cgi.params.delete('blogcategory')
		else
			member_cats = ( get00_member_cats( cat ) ) & index_catdays.keys
			# search pages when cat is undefinined
			if member_cats.empty?
				@cgi.params['q'] = [ cat ]
				@cgi.params.delete('blogcategory')
			end
		end
	end
	### end for blex ///////////////////////////////
	
	request = TDiary::Request.new( ENV, @cgi )
	status, headers, body = TDiary::Dispatcher.index.dispatch_cgi( request, @cgi )
	headers['type'] = headers.delete('Content-Type')
	TDiary::Dispatcher.send_headers( status, headers )
	TDiary::Dispatcher.send_body( body )
rescue Exception
	if @cgi then
		print @cgi.header( 'status' => CGI::HTTP_STATUS['SERVER_ERROR'], 'type' => 'text/html' )
	else
		print "Status: 500 Internal Server Error\n"
		print "Content-Type: text/html\n\n"
	end
	puts "<h1>500 Internal Server Error</h1>"
	puts "<pre>"
	puts CGI::escapeHTML( "#{$!} (#{$!.class})" )
	puts ""
	puts CGI::escapeHTML( $@.join( "\n" ) )
	puts "</pre>"
	puts "<div>#{' ' * 500}</div>"
end

# Local Variables:
# mode: ruby
# indent-tabs-mode: t
# tab-width: 3
# ruby-indent-level: 3
# End:
